@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: error.c,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: error.c,v 1.17 2005/08/04 04:55:45 aquamaniac Exp $
+    cvs         : $Id: error.c 957 2005-12-23 01:43:18Z aquamaniac $
     begin       : Tue Oct 02 2002
     copyright   : (C) 2002 by Martin Preuss
     email       : martin@libchipcard.de
@@ -78,8 +78,9 @@ GWEN_ERRORTYPEREGISTRATIONFORM *GWEN_ErrorType_new(){
 
 
 void GWEN_ErrorType_free(GWEN_ERRORTYPEREGISTRATIONFORM *f){
-  if (f)
-    free(f);
+  if (f) {
+    GWEN_FREE_OBJECT(f);
+  }
 }
 
 
