@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: plugindescr.c,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: plugindescr.c,v 1.21 2005/10/04 17:17:55 aquamaniac Exp $
+ cvs         : $Id: plugindescr.c 951 2005-12-05 18:01:55Z aquamaniac $
  begin       : Thu Apr 03 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -156,8 +156,14 @@ GWEN_PluginDescription_List2_freeAll_cb(GWEN_PLUGIN_DESCRIPTION *pd,
 void GWEN_PluginDescription_List2_freeAll(GWEN_PLUGIN_DESCRIPTION_LIST2 *pdl){
   GWEN_PluginDescription_List2_ForEach
     (pdl,
-     GWEN_PluginDescription_List2_freeAll_cb, 
+     GWEN_PluginDescription_List2_freeAll_cb,
      0);
+  /* The following line has been disabled to allow GnuCash 1.8.12 to work with
+   * this version of Gwen. As soon as the next Gnucash release appears we will
+   * enable this line again.
+   * Martin Preuss, 2005/12/05
+   GWEN_PluginDescription_List2_free(pdl);
+   */
 }
 
 
