@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: waitcallback.c,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: waitcallback.c,v 1.38 2005/08/09 15:04:03 aquamaniac Exp $
+ cvs         : $Id: waitcallback.c 994 2006-03-02 14:07:48Z aquamaniac $
  begin       : Sat Jan 17 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
@@ -487,12 +487,12 @@ void GWEN_WaitCallback_EnterWithText_u(const char *id,
         nctx->level=1;
       }
 
-      if (txt)
+      if (txt && nctx->originalCtx->logFn)
         /* inform the original context about the attachment */
-        ctx->originalCtx->logFn(ctx->originalCtx,
-                                GWEN_WAITCALLBACK_LEVEL_REUSED,
-                                GWEN_LoggerLevelNotice,
-                                txt);
+        nctx->originalCtx->logFn(nctx->originalCtx,
+                                 GWEN_WAITCALLBACK_LEVEL_REUSED,
+                                 GWEN_LoggerLevel_Notice,
+                                 txt);
     }
     else {
       DBG_DEBUG(GWEN_LOGDOMAIN, "Instantiating from callback \"%s\"", id);
@@ -519,6 +519,12 @@ void GWEN_WaitCallback_EnterWithText_u(const char *id,
     if (units)
       nctx->units=strdup(units);
     nctx->flags=flags;
+    if (GWEN__WaitCallback(nctx)==GWEN_WaitCallbackResult_Abort) {
+      DBG_WARN(GWEN_LOGDOMAIN,
+	       "Callback \"%s\" immediately wants to abort, "
+	       "please check your implementation.",
+	       nctx->id);
+    }
   }
 }
 
