@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: crypttoken_p.h,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: crypttoken_p.h,v 1.8 2005/09/09 11:41:33 aquamaniac Exp $
+    cvs         : $Id: crypttoken_p.h 988 2006-02-15 02:05:42Z aquamaniac $
     begin       : Wed Mar 16 2005
     copyright   : (C) 2005 by Martin Preuss
     email       : martin@libchipcard.de
@@ -63,6 +63,11 @@ struct GWEN_CRYPTTOKEN_CONTEXT {
   GWEN_CRYPTTOKEN_KEYINFO *encryptKeyInfo;
   GWEN_CRYPTTOKEN_KEYINFO *decryptKeyInfo;
   GWEN_CRYPTTOKEN_CRYPTINFO *cryptInfo;
+
+  GWEN_CRYPTTOKEN_KEYINFO *localAuthKeyInfo;
+  GWEN_CRYPTTOKEN_KEYINFO *remoteAuthKeyInfo;
+  GWEN_CRYPTTOKEN_SIGNINFO *authInfo;
+
 };
 
 
@@ -79,6 +84,7 @@ struct GWEN_CRYPTTOKEN {
   char *descriptiveName;
   GWEN_CRYPTTOKEN_DEVICE deviceType;
   GWEN_TYPE_UINT32 flags;
+  GWEN_TYPE_UINT32 modes;
 
   GWEN_CRYPTTOKEN_OPEN_FN openFn;
   GWEN_CRYPTTOKEN_CREATE_FN createFn;
@@ -90,6 +96,8 @@ struct GWEN_CRYPTTOKEN {
   GWEN_CRYPTTOKEN_VERIFY_FN verifyFn;
   GWEN_CRYPTTOKEN_ENCRYPT_FN encryptFn;
   GWEN_CRYPTTOKEN_DECRYPT_FN decryptFn;
+  GWEN_CRYPTTOKEN_AUTHSIGN_FN authSignFn;
+  GWEN_CRYPTTOKEN_AUTHVERIFY_FN authVerifyFn;
   GWEN_CRYPTTOKEN_READKEY_FN readKeyFn;
   GWEN_CRYPTTOKEN_WRITEKEY_FN writeKeyFn;
   GWEN_CRYPTTOKEN_READKEYSPEC_FN readKeySpecFn;
@@ -122,51 +130,64 @@ struct GWEN_CRYPTTOKEN_PLUGIN {
   GWEN_CRYPTTOKEN_PLUGIN_CREATETOKEN_FN createTokenFn;
   GWEN_CRYPTTOKEN_PLUGIN_CHECKTOKEN_FN checkTokenFn;
 };
-void GWEN_CryptToken_Plugin_FreeData(void *bp, void *p);
+static void GWEN_CryptToken_Plugin_FreeData(void *bp, void *p);
 
 
 
 typedef struct GWEN_CRYPTMANAGER GWEN_CRYPTMANAGER;
 struct GWEN_CRYPTMANAGER {
   GWEN_CRYPTMANAGER_GETPIN_FN getPinFn;
+  GWEN_CRYPTMANAGER_SETPINSTATUS_FN setPinStatusFn;
   GWEN_CRYPTMANAGER_BEGIN_ENTER_PIN_FN beginEnterPinFn;
   GWEN_CRYPTMANAGER_END_ENTER_PIN_FN endEnterPinFn;
   GWEN_CRYPTMANAGER_INSERT_TOKEN_FN insertTokenFn;
   GWEN_CRYPTMANAGER_INSERT_CORRECT_TOKEN_FN insertCorrectTokenFn;
   GWEN_CRYPTMANAGER_SHOW_MESSAGE_FN showMessageFn;
 };
-void GWEN_CryptManager_FreeData(void *bp, void *p);
+static void GWEN_CryptManager_FreeData(void *bp, void *p);
 
 
 
-GWEN_CRYPTTOKEN_CRYPTINFO *GWEN_CryptToken_CryptInfo_fromXml(GWEN_XMLNODE *n);
-GWEN_CRYPTTOKEN_SIGNINFO *GWEN_CryptToken_SignInfo_fromXml(GWEN_XMLNODE *n);
-GWEN_CRYPTTOKEN_KEYINFO *GWEN_CryptToken_KeyInfo_fromXml(GWEN_XMLNODE *n);
-GWEN_CRYPTTOKEN_CONTEXT *GWEN_CryptToken_Context__fromXml(GWEN_CRYPTTOKEN *ct,
-                                                          GWEN_XMLNODE *n);
+static GWEN_CRYPTTOKEN_CRYPTINFO*
+  GWEN_CryptToken_CryptInfo_fromXml(GWEN_XMLNODE *n);
+static GWEN_CRYPTTOKEN_SIGNINFO*
+  GWEN_CryptToken_SignInfo_fromXml(GWEN_XMLNODE *n);
+static GWEN_CRYPTTOKEN_KEYINFO*
+  GWEN_CryptToken_KeyInfo_fromXml(GWEN_XMLNODE *n);
+static GWEN_CRYPTTOKEN_CONTEXT*
+  GWEN_CryptToken_Context__fromXml(GWEN_CRYPTTOKEN *ct,
+                                   GWEN_XMLNODE *n);
 
-int GWEN_CryptToken__TransformFromBCD(unsigned char *buffer,
-                                      unsigned int bufLength,
-                                      unsigned int *pinLength);
+static int GWEN_CryptToken__TransformFromBCD(unsigned char *buffer,
+                                             unsigned int bufLength,
+                                             unsigned int *pinLength);
 
-int GWEN_CryptToken__TransformFromFPIN2(unsigned char *buffer,
-					unsigned int bufLength,
-                                        unsigned int *pinLength);
+static int GWEN_CryptToken__TransformFromFPIN2(unsigned char *buffer,
+                                               unsigned int bufLength,
+                                               unsigned int *pinLength);
 
-int GWEN_CryptToken__TransformFromBin(unsigned char *buffer,
-                                      unsigned int bufLength,
-                                      unsigned int *pinLength);
+static int GWEN_CryptToken__TransformFromBin(unsigned char *buffer,
+                                             unsigned int bufLength,
+                                             unsigned int *pinLength);
 
-int GWEN_CryptToken__TransformToBCD(unsigned char *buffer,
-				    unsigned int bufLength,
-                                    unsigned int *pinLength);
-int GWEN_CryptToken__TransformToFPIN2(unsigned char *buffer,
-				      unsigned int bufLength,
-                                      unsigned int *pinLength);
-int GWEN_CryptToken__TransformToBin(unsigned char *buffer,
-                                    unsigned int bufLength,
-                                    unsigned int *pinLength);
+static int GWEN_CryptToken__TransformToBCD(unsigned char *buffer,
+                                           unsigned int bufLength,
+                                           unsigned int *pinLength);
+static int GWEN_CryptToken__TransformToFPIN2(unsigned char *buffer,
+                                             unsigned int bufLength,
+                                             unsigned int *pinLength);
+static int GWEN_CryptToken__TransformToBin(unsigned char *buffer,
+                                           unsigned int bufLength,
+                                           unsigned int *pinLength);
 
+static int GWEN_CryptToken__matchKey(const GWEN_CRYPTTOKEN_KEYINFO *pki,
+                                     GWEN_CRYPTTOKEN_KEYINFO *ki,
+                                     const char *name);
+
+static int
+  GWEN_CryptToken__matchContext(GWEN_CRYPTTOKEN *ct,
+                                const GWEN_CRYPTTOKEN_CONTEXT *pattern,
+                                GWEN_CRYPTTOKEN_CONTEXT *ctx);
 
 
 
