@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: bio_buffer.c,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: bio_buffer.c,v 1.5 2005/08/04 04:55:46 aquamaniac Exp $
+ cvs         : $Id: bio_buffer.c 957 2005-12-23 01:43:18Z aquamaniac $
  begin       : Fri Feb 07 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -61,7 +61,7 @@ void GWEN_BufferedIO_Buffer_Table__free(GWEN_BUFFEREDIO_BUFFER *bft) {
   if (bft) {
     if (bft->own)
       GWEN_Buffer_free(bft->buffer);
-    free(bft);
+    GWEN_FREE_OBJECT(bft);
   }
 }
 
