@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: bufferedio.c,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: bufferedio.c,v 1.33 2005/07/29 02:03:55 aquamaniac Exp $
+ cvs         : $Id: bufferedio.c 995 2006-03-03 11:54:36Z cstim $
  begin       : Fri Feb 07 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -141,7 +141,7 @@ void GWEN_BufferedIO_free(GWEN_BUFFEREDIO *bt){
     bt->readerBuffer=0;
     free(bt->writerBuffer);
     bt->writerBuffer=0;
-    free(bt);
+    GWEN_FREE_OBJECT(bt);
   }
 }
 
@@ -231,7 +231,14 @@ int GWEN_BufferedIO__FillReadBuffer(GWEN_BUFFEREDIO *bt){
 
 int GWEN_BufferedIO_PeekChar(GWEN_BUFFEREDIO *bt){
   assert(bt);
-  assert(bt->readerBuffer);
+
+  /* Extend the assertion: If it fails, print a more verbose error
+     message to help programmers who are inexperienced with the
+     BUFFEREDIO. */
+  if (!bt->readerBuffer) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "BufferedIO has not yet been assigned a reading buffer for reading; probably assign one by GWEN_BufferedIO_SetReadBuffer(bio,0,length).");
+    assert(bt->readerBuffer);
+  }
 
   /* do some fast checks */
   if (bt->readerError) {
@@ -616,7 +623,7 @@ void GWEN_BufferedIO_SetLineMode(GWEN_BUFFEREDIO *dm,
 
 
 
-GWEN_BUFFEREDIOLINEMODE GWEN_BufferedIO_GetLineMode(GWEN_BUFFEREDIO *dm){
+GWEN_BUFFEREDIOLINEMODE GWEN_BufferedIO_GetLineMode(const GWEN_BUFFEREDIO *dm){
   assert(dm);
   return dm->lineMode;
 }
@@ -630,7 +637,7 @@ void GWEN_BufferedIO_SetTimeout(GWEN_BUFFEREDIO *dm, int timeout){
 
 
 
-int GWEN_BufferedIO_GetTimeout(GWEN_BUFFEREDIO *dm){
+int GWEN_BufferedIO_GetTimeout(const GWEN_BUFFEREDIO *dm){
   assert(dm);
   return dm->timeout;
 }
@@ -815,10 +822,10 @@ GWEN_ERRORCODE GWEN_BufferedIO_ReadRawForced(GWEN_BUFFEREDIO *bt,
 
 
 GWEN_ERRORCODE GWEN_BufferedIO_WriteRawForced(GWEN_BUFFEREDIO *bt,
-                                              char *buffer,
+                                              const char *buffer,
                                               unsigned int *bsize){
   GWEN_ERRORCODE err;
-  char *lbuffer;
+  const char *lbuffer;
   unsigned int lsize;
   unsigned int bytesWritten;
   GWEN_WAITCALLBACK_RESULT cbres;
