@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: directory.h,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: directory.h,v 1.16 2005/08/17 19:52:39 aquamaniac Exp $
+    cvs         : $Id: directory.h 834 2005-08-17 19:52:39Z aquamaniac $
     begin       : Tue Dec 03 2002
     copyright   : (C) 2002 by Martin Preuss
     email       : martin@libchipcard.de
