@@ -1,7 +1,7 @@
 /***************************************************************************
-  $RCSfile: process_p.h,v $
+  $RCSfile$
                              -------------------
-    cvs         : $Id: process_p.h,v 1.4 2004/04/08 10:37:38 aquamaniac Exp $
+    cvs         : $Id: process_p.h 303 2004-04-08 10:37:38Z aquamaniac $
     begin       : Sun Dec 28 2003
     copyright   : (C) 2003 by Martin Preuss
     email       : martin@libchipcard.de
