@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: text.c,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: text.c,v 1.38 2005/08/27 06:30:27 aquamaniac Exp $
+ cvs         : $Id: text.c 1014 2006-03-25 10:33:27Z cstim $
  begin       : Sat Jun 28 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -1606,7 +1606,7 @@ int GWEN_Text_UnescapeXmlToBuffer(const char *src, GWEN_BUFFER *buf) {
 	unsigned char num=0;
 
 	src++;
-	while(isdigit(*src)) {
+	while(isdigit((int)*src)) {
 	  num*=10;
 	  num+=(*src)-'0';
 	  src++;
