@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: tlv_p.h,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: tlv_p.h,v 1.1 2004/08/06 08:14:56 aquamaniac Exp $
+    cvs         : $Id: tlv_p.h 490 2004-08-06 08:14:56Z aquamaniac $
     begin       : Sun Jun 13 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
