@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: xml.c,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: xml.c,v 1.59 2005/07/23 09:32:08 aquamaniac Exp $
+ cvs         : $Id: xml.c 999 2006-03-15 10:03:08Z aquamaniac $
  begin       : Sat Jun 28 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -53,6 +53,7 @@
 
 
 
+GWEN_LIST_FUNCTIONS(GWEN_XMLNODE, GWEN_XMLNode)
 GWEN_LIST2_FUNCTIONS(GWEN_XMLNODE, GWEN_XMLNode)
 
 
@@ -62,9 +63,9 @@ GWEN_XMLPROPERTY *GWEN_XMLProperty_new(const char *name, const char *value){
 
   GWEN_NEW_OBJECT(GWEN_XMLPROPERTY, p);
   if (name)
-    p->name=strdup(name);
+    p->name=GWEN_Memory_strdup(name);
   if (value)
-    p->value=strdup(value);
+    p->value=GWEN_Memory_strdup(value);
   return p;
 }
 
@@ -72,9 +73,9 @@ GWEN_XMLPROPERTY *GWEN_XMLProperty_new(const char *name, const char *value){
 
 void GWEN_XMLProperty_free(GWEN_XMLPROPERTY *p){
   if (p) {
-    free(p->name);
-    free(p->value);
-    free(p);
+    GWEN_Memory_dealloc(p->name);
+    GWEN_Memory_dealloc(p->value);
+    GWEN_FREE_OBJECT(p);
   }
 }
 
@@ -118,23 +119,25 @@ void GWEN_XMLProperty_freeAll(GWEN_XMLPROPERTY *p) {
 GWEN_XMLNODE *GWEN_XMLNode_new(GWEN_XMLNODE_TYPE t, const char *data){
   GWEN_XMLNODE *n;
 
-  n=(GWEN_XMLNODE *)malloc(sizeof(GWEN_XMLNODE));
-  assert(n);
-  memset(n,0,sizeof(GWEN_XMLNODE));
+  GWEN_NEW_OBJECT(GWEN_XMLNODE, n);
+  GWEN_LIST_INIT(GWEN_XMLNODE, n);
   n->type=t;
+  n->children=GWEN_XMLNode_List_new();
+  n->headers=GWEN_XMLNode_List_new();
   if (data)
-    n->data=strdup(data);
+    n->data=GWEN_Memory_strdup(data);
   return n;
 }
 
 
 void GWEN_XMLNode_free(GWEN_XMLNODE *n){
   if (n) {
+    GWEN_LIST_FINI(GWEN_XMLNODE, n);
     GWEN_XMLProperty_freeAll(n->properties);
-    free(n->data);
-    GWEN_XMLNode_freeAll(n->child);
-    GWEN_XMLNode_freeAll(n->header);
-    free(n);
+    GWEN_Memory_dealloc(n->data);
+    GWEN_XMLNode_List_free(n->headers);
+    GWEN_XMLNode_List_free(n->children);
+    GWEN_FREE_OBJECT(n);
   }
 }
 
@@ -143,7 +146,7 @@ void GWEN_XMLNode_freeAll(GWEN_XMLNODE *n){
   while(n) {
     GWEN_XMLNODE *next;
 
-    next=n->next;
+    next=GWEN_XMLNode_List_Next(n);
     GWEN_XMLNode_free(n);
     n=next;
   } /* while */
@@ -168,29 +171,25 @@ GWEN_XMLNODE *GWEN_XMLNode_dup(const GWEN_XMLNODE *n){
   } /* while */
 
   /* duplicate children */
-  cn=n->child;
+  cn=GWEN_XMLNode_List_First(n->children);
   while(cn) {
     ncn=GWEN_XMLNode_dup(cn);
-    GWEN_XMLNode_add(ncn, &(nn->child));
-    ncn->parent=nn;
-    cn=cn->next;
+    GWEN_XMLNode_AddChild(nn, ncn);
+    cn=GWEN_XMLNode_Next(cn);
+  } /* while */
+
+  /* duplicate headers */
+  cn=GWEN_XMLNode_List_First(n->headers);
+  while(cn) {
+    ncn=GWEN_XMLNode_dup(cn);
+    GWEN_XMLNode_AddHeader(nn, ncn);
+    cn=GWEN_XMLNode_Next(cn);
   } /* while */
 
   return nn;
 }
 
 
-void GWEN_XMLNode_add(GWEN_XMLNODE *n, GWEN_XMLNODE **head){
-  GWEN_LIST_ADD(GWEN_XMLNODE, n, head);
-}
-
-
-void GWEN_XMLNode_del(GWEN_XMLNODE *n, GWEN_XMLNODE **head){
-  GWEN_LIST_DEL(GWEN_XMLNODE, n, head);
-  n->parent=0;
-}
-
-
 
 GWEN_XMLNODE *GWEN_XMLNode_fromString(const char *s,
                                       int len,
@@ -255,9 +254,9 @@ void GWEN_XMLNode__SetProperty(GWEN_XMLNODE *n,
   } /* while */
 
   if (p) {
-    free(p->value);
+    GWEN_Memory_dealloc(p->value);
     if (value)
-      p->value=strdup(value);
+      p->value=GWEN_Memory_strdup(value);
     else
       p->value=0;
   }
@@ -312,9 +311,9 @@ const char *GWEN_XMLNode_GetData(const GWEN_XMLNODE *n){
 
 void GWEN_XMLNode_SetData(GWEN_XMLNODE *n, const char *data){
   assert(n);
-  free(n->data);
+  GWEN_Memory_dealloc(n->data);
   if (data)
-    n->data=strdup(data);
+    n->data=GWEN_Memory_strdup(data);
   else
     n->data=0;
 }
@@ -322,7 +321,7 @@ void GWEN_XMLNode_SetData(GWEN_XMLNODE *n, const char *data){
 
 GWEN_XMLNODE *GWEN_XMLNode_GetChild(const GWEN_XMLNODE *n){
   assert(n);
-  return n->child;
+  return GWEN_XMLNode_List_First(n->children);
 }
 
 
@@ -334,7 +333,7 @@ GWEN_XMLNODE *GWEN_XMLNode_GetParent(const GWEN_XMLNODE *n){
 
 void GWEN_XMLNode_AddChild(GWEN_XMLNODE *n, GWEN_XMLNODE *child){
   assert(n);
-  GWEN_XMLNode_add(child, &(n->child));
+  GWEN_XMLNode_List_Add(child, n->children);
   child->parent=n;
 }
 
@@ -493,11 +492,11 @@ void GWEN_XMLNode_AddChildrenOnly(GWEN_XMLNODE *n, GWEN_XMLNODE *nn,
   assert(n);
   assert(nn);
 
-  ch=nn->child;
+  ch=GWEN_XMLNode_GetChild(nn);
   while(ch) {
     GWEN_XMLNODE *nc;
 
-    nc=ch->next;
+    nc=GWEN_XMLNode_Next(ch);
     if (!copythem) {
       GWEN_XMLNode_UnlinkChild(nn, ch);
       GWEN_XMLNode_AddChild(n, ch);
@@ -665,8 +664,7 @@ int GWEN_XML_ReadBIO(GWEN_XMLNODE *n,
           newNode=GWEN_XMLNode_new(GWEN_XMLNodeTypeComment,
                                    GWEN_Buffer_GetStart(tbuf));
           GWEN_Buffer_free(tbuf);
-          GWEN_XMLNode_add(newNode, &(n->child));
-          newNode->parent=n;
+          GWEN_XMLNode_AddChild(n, newNode);
           DBG_DEBUG(GWEN_LOGDOMAIN, "Added comment: \"%s\"",
                     GWEN_Buffer_GetStart(bufComment));
         }
@@ -729,7 +727,7 @@ int GWEN_XML_ReadBIO(GWEN_XMLNODE *n,
             const char *iname;
 
             iname=0;
-            inametag=n->child;
+            inametag=GWEN_XMLNode_GetChild(n);
             if (inametag) {
               /* found a child */
               if (inametag->type==GWEN_XMLNodeTypeData) {
@@ -772,11 +770,11 @@ int GWEN_XML_ReadBIO(GWEN_XMLNODE *n,
               /* unlink <INCLUDE> tag */
               GWEN_XMLNode_UnlinkChild(nparent, n);
               /* move all children of newRoot to nparent */
-              itag=newRoot->child;
+              itag=GWEN_XMLNode_GetChild(newRoot);
               while(itag) {
                 GWEN_XMLNODE *nextc;
 
-                nextc=itag->next;
+                nextc=GWEN_XMLNode_Next(itag);
                 if (flags & GWEN_XML_FLAGS_INCLUDE_TO_TOPLEVEL) {
                   GWEN_XMLNODE *tl;
 
@@ -1139,8 +1137,7 @@ int GWEN_XML_ReadBIO(GWEN_XMLNODE *n,
                   GWEN_Buffer_free(bufTagName);
                   return -1;
                 }
-                GWEN_XMLNode_add(newNode, &(n->child));
-                newNode->parent=n;
+                GWEN_XMLNode_AddChild(n, newNode);
                 DBG_DEBUG(GWEN_LOGDOMAIN, "Added node \"%s\"", newNode->data);
               }
             }
@@ -1154,8 +1151,7 @@ int GWEN_XML_ReadBIO(GWEN_XMLNODE *n,
                 GWEN_Buffer_free(bufTagName);
                 return -1;
               }
-              GWEN_XMLNode_add(newNode, &(n->child));
-              newNode->parent=n;
+              GWEN_XMLNode_AddChild(n, newNode);
               DBG_DEBUG(GWEN_LOGDOMAIN, "Added node \"%s\"", newNode->data);
             }
             if (!isEndTag && !simpleTag) {
@@ -1210,7 +1206,7 @@ int GWEN_XML_ReadBIO(GWEN_XMLNODE *n,
       GWEN_Buffer_free(tbuf);
       DBG_DEBUG(GWEN_LOGDOMAIN, "Added data \"%s\"",
                 GWEN_Buffer_GetStart(bufData));
-      GWEN_XMLNode_add(newNode, &(n->child));
+      GWEN_XMLNode_AddChild(n, newNode);
       newNode->parent=n;
 
       GWEN_Buffer_free(bufData);
@@ -1257,7 +1253,7 @@ int GWEN_XML_ReadFileInt(GWEN_XMLNODE *n,
     if (path)
       i+=strlen(path)+1;
     i+=strlen(file)+1;
-    fullname=(char*)malloc(i);
+    fullname=(char*)GWEN_Memory_malloc(i);
     assert(fullname);
     fullname[0]=0;
     if (path) {
@@ -1286,7 +1282,7 @@ int GWEN_XML_ReadFileInt(GWEN_XMLNODE *n,
         DBG_INFO(GWEN_LOGDOMAIN, "open(%s): %s",
                  fullname,
                  strerror(errno));
-        free(fullname);
+        GWEN_Memory_dealloc(fullname);
         fullname=0;
         gotcha=0;
 
@@ -1309,7 +1305,7 @@ int GWEN_XML_ReadFileInt(GWEN_XMLNODE *n,
           i=0;
           i+=strlen(sp)+1;
           i+=strlen(file)+1;
-          fullname=(char*)malloc(i);
+          fullname=(char*)GWEN_Memory_malloc(i);
           assert(fullname);
           fullname[0]=0;
           strcpy(fullname, sp);
@@ -1329,7 +1325,7 @@ int GWEN_XML_ReadFileInt(GWEN_XMLNODE *n,
                      strerror(errno));
           }
 
-          free(fullname);
+          GWEN_Memory_dealloc(fullname);
           fullname=0;
           se=GWEN_StringListEntry_Next(se);
         } /* while */
@@ -1347,7 +1343,7 @@ int GWEN_XML_ReadFileInt(GWEN_XMLNODE *n,
     /* create full name from file name only, since it is absolute */
     i=0;
     i+=strlen(file)+1;
-    fullname=(char*)malloc(i);
+    fullname=(char*)GWEN_Memory_malloc(i);
     assert(fullname);
     fullname[0]=0;
     strcat(fullname, file);
@@ -1359,7 +1355,7 @@ int GWEN_XML_ReadFileInt(GWEN_XMLNODE *n,
     DBG_INFO(GWEN_LOGDOMAIN, "open(%s): %s",
 	     fullname,
 	     strerror(errno));
-    free(fullname);
+    GWEN_Memory_dealloc(fullname);
     return -1;
   }
 
@@ -1377,14 +1373,14 @@ int GWEN_XML_ReadFileInt(GWEN_XMLNODE *n,
   while(!GWEN_BufferedIO_CheckEOF(dm)) {
     if (GWEN_XML_ReadBIO(n, dm, flags, path, sl, GWEN_XML_ReadFileInt)) {
       DBG_ERROR(GWEN_LOGDOMAIN, "Error parsing");
-      free(fullname);
+      GWEN_Memory_dealloc(fullname);
       GWEN_BufferedIO_Close(dm);
       GWEN_BufferedIO_free(dm);
       return -1;
     }
   } /* while */
 
-  free(fullname);
+  GWEN_Memory_dealloc(fullname);
   GWEN_BufferedIO_Close(dm);
   GWEN_BufferedIO_free(dm);
 
@@ -1401,7 +1397,7 @@ GWEN_XMLNODE_TYPE GWEN_XMLNode_GetType(const GWEN_XMLNODE *n){
 
 GWEN_XMLNODE *GWEN_XMLNode_Next(const GWEN_XMLNODE *n) {
   assert(n);
-  return n->next;
+  return GWEN_XMLNode_List_Next(n);
 }
 
 
@@ -1443,10 +1439,10 @@ void GWEN_XMLNode_Dump(const GWEN_XMLNODE *n, FILE *f, int ind) {
 
     fprintf(f, ">\n");
     if (!simpleTag) {
-      c=n->child;
+      c=GWEN_XMLNode_GetChild(n);
       while(c) {
         GWEN_XMLNode_Dump(c, f, ind+2);
-        c=c->next;
+        c=GWEN_XMLNode_Next(c);
       }
       for(i=0; i<ind; i++)
         fprintf(f, " ");
@@ -1481,13 +1477,13 @@ GWEN_XMLNODE *GWEN_XMLNode_FindNode(const GWEN_XMLNODE *node,
   assert(node);
   assert(data);
 
-  n=node->child;
+  n=GWEN_XMLNode_GetChild(node);
   while(n) {
     if (n->type==t)
       if (n->data)
-	if (strcasecmp(n->data, data)==0)
+        if (strcasecmp(n->data, data)==0)
 	  break;
-    n=n->next;
+    n=GWEN_XMLNode_Next(n);
   } /* while */
 
   if (!n) {
@@ -1503,26 +1499,15 @@ GWEN_XMLNODE *GWEN_XMLNode_FindNode(const GWEN_XMLNODE *node,
 void GWEN_XMLNode_UnlinkChild(GWEN_XMLNODE *n, GWEN_XMLNODE *child){
   assert(n);
   assert(child);
-  GWEN_XMLNode_del(child, &(n->child));
-  child->next=0;
+  GWEN_XMLNode_List_Del(child);
   child->parent=0;
 }
 
 
 
 void GWEN_XMLNode_RemoveChildren(GWEN_XMLNODE *n){
-  GWEN_XMLNODE *cn;
-
   assert(n);
-  cn=n->child;
-  while(cn) {
-    GWEN_XMLNODE *ncn;
-
-    ncn=cn->next;
-    GWEN_XMLNode_free(cn);
-    cn=ncn;
-  } /* while */
-  n->child=0;
+  GWEN_XMLNode_List_Clear(n->children);
 }
 
 
@@ -1550,10 +1535,10 @@ void GWEN_XMLNode_CopyProperties(GWEN_XMLNODE *tn,
 	/* property already exists */
 	if (overwrite) {
           /* overwrite old property */
-	  free(tp->value);
+	  GWEN_Memory_dealloc(tp->value);
 	  tp->value=0;
 	  if (sp->value)
-	    tp->value=strdup(sp->value);
+	    tp->value=GWEN_Memory_strdup(sp->value);
 	}
 	break;
       }
@@ -1577,11 +1562,11 @@ GWEN_XMLNODE *GWEN_XMLNode_GetFirstOfType(const GWEN_XMLNODE *n,
   GWEN_XMLNODE *nn;
 
   assert(n);
-  nn=n->child;
+  nn=GWEN_XMLNode_GetChild(n);
   while(nn) {
     if (nn->type==t)
       return nn;
-    nn=nn->next;
+    nn=GWEN_XMLNode_Next(nn);
   } /* while */
   return 0;
 }
@@ -1594,7 +1579,7 @@ GWEN_XMLNODE *GWEN_XMLNode_GetNextOfType(const GWEN_XMLNODE *n,
   while(n) {
     if (n->type==t)
       return (GWEN_XMLNODE *)n;
-    n=n->next;
+    n=GWEN_XMLNode_Next(n);
   } /* while */
   return 0;
 }
@@ -1608,9 +1593,12 @@ GWEN_XMLNODE *GWEN_XMLNode_GetFirstTag(const GWEN_XMLNODE *n){
 
 
 GWEN_XMLNODE *GWEN_XMLNode_GetNextTag(const GWEN_XMLNODE *n){
-  if (!n->next)
+  GWEN_XMLNODE *next;
+
+  next=GWEN_XMLNode_Next(n);
+  if (!next)
     return 0;
-  return GWEN_XMLNode_GetNextOfType(n->next, GWEN_XMLNodeTypeTag);
+  return GWEN_XMLNode_GetNextOfType(next, GWEN_XMLNodeTypeTag);
 }
 
 
@@ -1622,9 +1610,12 @@ GWEN_XMLNODE *GWEN_XMLNode_GetFirstData(const GWEN_XMLNODE *n){
 
 
 GWEN_XMLNODE *GWEN_XMLNode_GetNextData(const GWEN_XMLNODE *n){
-  if (!n->next)
+  GWEN_XMLNODE *next;
+
+  next=GWEN_XMLNode_Next(n);
+  if (!next)
     return 0;
-  return GWEN_XMLNode_GetNextOfType(n->next, GWEN_XMLNodeTypeData);
+  return GWEN_XMLNode_GetNextOfType(next, GWEN_XMLNodeTypeData);
 }
 
 
@@ -1733,7 +1724,7 @@ GWEN_XMLNODE_PATH *GWEN_XMLNode_Path_dup(const GWEN_XMLNODE_PATH *np){
 
 
 void GWEN_XMLNode_Path_free(GWEN_XMLNODE_PATH *np){
-  free(np);
+  GWEN_FREE_OBJECT(np);
 }
 
 
@@ -1824,6 +1815,7 @@ int GWEN_XMLNode__WriteToStream(const GWEN_XMLNODE *n,
         return -1;
       }
       err=GWEN_BufferedIO_Write(bio, GWEN_Buffer_GetStart(tbuf));
+      GWEN_Buffer_free(tbuf);
     }
     else
       err=GWEN_BufferedIO_Write(bio, "UNKNOWN");
@@ -1876,7 +1868,7 @@ int GWEN_XMLNode__WriteToStream(const GWEN_XMLNODE *n,
           return -1;
         }
   
-        if (p->value) {
+        if (p->value && *(p->value)) {
           err=GWEN_BufferedIO_Write(bio, "=\"");
           if (!GWEN_Error_IsOk(err)) {
             DBG_ERROR_ERR(GWEN_LOGDOMAIN, err);
@@ -1935,7 +1927,7 @@ int GWEN_XMLNode__WriteToStream(const GWEN_XMLNODE *n,
       }
     }
 
-    if (n->child==0) {
+    if (GWEN_XMLNode_GetChild(n)==0) {
       if (n->data[0]=='?')
 	err=GWEN_BufferedIO_Write(bio, ">");
       else
@@ -1957,7 +1949,7 @@ int GWEN_XMLNode__WriteToStream(const GWEN_XMLNODE *n,
       }
 
       if (!simpleTag) {
-        c=n->child;
+        c=GWEN_XMLNode_GetChild(n);
         while(c) {
           if (GWEN_XMLNode__WriteToStream(c, bio, flags, ind+2)) {
             DBG_INFO(GWEN_LOGDOMAIN, "Error writing tag \"%s\"", n->data);
@@ -1972,7 +1964,7 @@ int GWEN_XMLNode__WriteToStream(const GWEN_XMLNODE *n,
               return -1;
             }
           }
-          c=c->next;
+          c=GWEN_XMLNode_Next(c);
         }
         if ((flags & GWEN_XML_FLAGS_INDENT) &&
             GWEN_BufferedIO_GetLinePos(bio)==0){
@@ -1999,6 +1991,7 @@ int GWEN_XMLNode__WriteToStream(const GWEN_XMLNODE *n,
             return -1;
           }
           err=GWEN_BufferedIO_Write(bio, GWEN_Buffer_GetStart(tbuf));
+          GWEN_Buffer_free(tbuf);
         }
         else
           err=GWEN_BufferedIO_Write(bio, "UNKNOWN");
@@ -2023,7 +2016,7 @@ int GWEN_XMLNode__WriteToStream(const GWEN_XMLNODE *n,
     } /* if tag has children */
   } /* if tag */
   else if (n->type==GWEN_XMLNodeTypeData) {
-    if (n->data) {
+    if (n->data && *(n->data)) {
       GWEN_BUFFER *tbuf;
 
       tbuf=GWEN_Buffer_new(0, strlen(n->data), 0, 1);
@@ -2052,7 +2045,7 @@ int GWEN_XMLNode__WriteToStream(const GWEN_XMLNODE *n,
         return -1;
       }
 
-      if (n->data) {
+      if (n->data && *(n->data)) {
         GWEN_BUFFER *tbuf;
   
         tbuf=GWEN_Buffer_new(0, strlen(n->data), 0, 1);
@@ -2088,15 +2081,24 @@ int GWEN_XMLNode_WriteToStream(const GWEN_XMLNODE *n,
                                GWEN_BUFFEREDIO *bio,
                                GWEN_TYPE_UINT32 flags){
   const GWEN_XMLNODE *nn;
+  const GWEN_XMLNODE *nchild;
+  const GWEN_XMLNODE *nheader;
 
-  if (n->header && (flags & GWEN_XML_FLAGS_HANDLE_HEADERS)) {
+  nchild=GWEN_XMLNode_GetChild(n);
+  nheader=GWEN_XMLNode_GetHeader(n);
+
+  if (nheader && (flags & GWEN_XML_FLAGS_HANDLE_HEADERS)) {
     GWEN_TYPE_UINT32 lflags;
+
     lflags=flags & ~GWEN_XML_FLAGS_HANDLE_HEADERS;
-    nn=n->header;
+    nn=nheader;
     while(nn) {
+      const GWEN_XMLNODE *next;
+
       if (GWEN_XMLNode__WriteToStream(nn, bio, lflags, 0))
         return -1;
-      if (nn->next) {
+      next=GWEN_XMLNode_Next(nn);
+      if (next) {
         GWEN_ERRORCODE err;
 
         err=GWEN_BufferedIO_WriteLine(bio, "");
@@ -2106,9 +2108,10 @@ int GWEN_XMLNode_WriteToStream(const GWEN_XMLNODE *n,
         }
       }
 
-      nn=nn->next;
+      nn=next;
     }
-    if (n->child) {
+
+    if (nchild) {
       GWEN_ERRORCODE err;
 
       err=GWEN_BufferedIO_WriteLine(bio, "");
@@ -2119,11 +2122,14 @@ int GWEN_XMLNode_WriteToStream(const GWEN_XMLNODE *n,
     }
   }
 
-  nn=n->child;
+  nn=nchild;
   while(nn) {
+    const GWEN_XMLNODE *next;
+
     if (GWEN_XMLNode__WriteToStream(nn, bio, flags, 0))
       return -1;
-    if (nn->next) {
+    next=GWEN_XMLNode_Next(nn);
+    if (next) {
       GWEN_ERRORCODE err;
 
       err=GWEN_BufferedIO_WriteLine(bio, "");
@@ -2133,7 +2139,7 @@ int GWEN_XMLNode_WriteToStream(const GWEN_XMLNODE *n,
       }
     }
 
-    nn=nn->next;
+    nn=next;
   } /* while */
 
   return 0;
@@ -2442,12 +2448,13 @@ int GWEN_XMLNode_GetXPath(const GWEN_XMLNODE *n1,
 
 void* GWEN_XMLNode_HandlePath(const char *entry,
                               void *data,
+                              int idx,
                               GWEN_TYPE_UINT32 flags) {
   GWEN_XMLNODE *n;
   GWEN_XMLNODE *nn;
-  int idx;
   char *tag;
   char *p;
+  int i;
 
   n=(GWEN_XMLNODE*)data;
 
@@ -2472,8 +2479,8 @@ void* GWEN_XMLNode_HandlePath(const char *entry,
     return n;
   }
 
-  idx=1;
-  tag=strdup(entry);
+  idx=0;
+  tag=GWEN_Memory_strdup(entry);
   assert(tag);
   p=strchr(tag, '[');
   if (p) {
@@ -2482,13 +2489,13 @@ void* GWEN_XMLNode_HandlePath(const char *entry,
     if (1!=sscanf(p, "%d]", &idx)) {
       DBG_ERROR(GWEN_LOGDOMAIN, "Bad path entry \"%s\" (bad index)",
                 entry);
-      free(tag);
+      GWEN_Memory_dealloc(tag);
       return 0;
     }
-    if (idx<1) {
+    if (idx<0) {
       DBG_ERROR(GWEN_LOGDOMAIN, "Bad index %d in path element \"%s\"",
                 idx, entry);
-      free(tag);
+      GWEN_Memory_dealloc(tag);
       return 0;
     }
   }
@@ -2508,37 +2515,30 @@ void* GWEN_XMLNode_HandlePath(const char *entry,
     /* simply create the new variable/group */
     if (flags & GWEN_PATH_FLAGS_VARIABLE) {
       /* not allowed for now */
-      free(tag);
+      GWEN_Memory_dealloc(tag);
       return 0;
     }
     else {
-      if (idx!=1) {
+      if (idx!=0) {
         DBG_ERROR(GWEN_LOGDOMAIN,
                   "Can not create tag with index!=1 (%s)", entry);
-        free(tag);
+        GWEN_Memory_dealloc(tag);
         return 0;
       }
       DBG_VERBOUS(GWEN_LOGDOMAIN,
                   "Unconditionally creating tag \"%s\"", entry);
       nn=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, tag);
       GWEN_XMLNode_AddChild(n, nn);
-      free(tag);
+      GWEN_Memory_dealloc(tag);
       return nn;
     }
   }
 
   /* find the node */
-  if (flags & GWEN_PATH_FLAGS_VARIABLE) {
-    nn=0;
-  }
-  else {
-    int i;
-
-    i=idx;
-    nn=GWEN_XMLNode_FindFirstTag(n, tag, 0, 0);
-    while(--i) {
-      nn=GWEN_XMLNode_FindNextTag(nn, tag, 0, 0);
-    }
+  i=idx;
+  nn=GWEN_XMLNode_FindFirstTag(n, tag, 0, 0);
+  while(nn && --i) {
+    nn=GWEN_XMLNode_FindNextTag(nn, tag, 0, 0);
   }
 
   if (!nn) {
@@ -2548,29 +2548,22 @@ void* GWEN_XMLNode_HandlePath(const char *entry,
          (flags & GWEN_PATH_FLAGS_PATHMUSTEXIST)) ||
         (flags & GWEN_PATH_FLAGS_NAMEMUSTEXIST)
        ) {
-      if (flags & GWEN_PATH_FLAGS_VARIABLE) {
-        DBG_VERBOUS(GWEN_LOGDOMAIN,
-                    "Variable \"%s\" does not exist", entry);
-      }
-      else {
-        DBG_VERBOUS(GWEN_LOGDOMAIN,
-                    "Tag \"%s\" does not exist", entry);
-      }
-      free(tag);
+      DBG_VERBOUS(GWEN_LOGDOMAIN,
+                  "Tag \"%s\" does not exist", entry);
+      GWEN_Memory_dealloc(tag);
       return 0;
     }
     /* create the new variable/group */
-    if (flags & GWEN_PATH_FLAGS_VARIABLE) {
-      /* variable not allowed */
-      free(tag);
+    if (idx!=0) {
+      DBG_ERROR(GWEN_LOGDOMAIN,
+                "Can not create tag with index!=1 (%s)", entry);
+      GWEN_Memory_dealloc(tag);
       return 0;
     }
-    else {
-      DBG_VERBOUS(GWEN_LOGDOMAIN,
-                  "Tag \"%s\" not found, creating", entry);
-      nn=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, tag);
-      GWEN_XMLNode_AddChild(n, nn);
-    }
+    DBG_VERBOUS(GWEN_LOGDOMAIN,
+                "Tag \"%s\" not found, creating", entry);
+    nn=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, tag);
+    GWEN_XMLNode_AddChild(n, nn);
   } /* if node not found */
   else {
     /* node does exist, check whether this is ok */
@@ -2581,12 +2574,12 @@ void* GWEN_XMLNode_HandlePath(const char *entry,
          (flags & GWEN_PATH_FLAGS_PATHMUSTNOTEXIST))
        ) {
       DBG_VERBOUS(GWEN_LOGDOMAIN, "Entry \"%s\" already exists", entry);
-      free(tag);
+      GWEN_Memory_dealloc(tag);
       return 0;
     }
   }
 
-  free(tag);
+  GWEN_Memory_dealloc(tag);
   return nn;
 }
 
@@ -2595,17 +2588,17 @@ void* GWEN_XMLNode_HandlePath(const char *entry,
 GWEN_XMLNODE *GWEN_XMLNode_GetNodeByXPath(GWEN_XMLNODE *n,
                                           const char *path,
                                           GWEN_TYPE_UINT32 flags){
-  return (GWEN_XMLNODE*)GWEN_Path_Handle(path,
-                                         (void*)n,
-                                         flags,
-                                         GWEN_XMLNode_HandlePath);
+  return (GWEN_XMLNODE*)GWEN_Path_HandleWithIdx(path,
+                                                (void*)n,
+                                                flags,
+                                                GWEN_XMLNode_HandlePath);
 }
 
 
 
 GWEN_XMLNODE *GWEN_XMLNode_GetHeader(const GWEN_XMLNODE *n){
   assert(n);
-  return n->header;
+  return GWEN_XMLNode_List_First(n->headers);
 }
 
 
@@ -2613,7 +2606,7 @@ GWEN_XMLNODE *GWEN_XMLNode_GetHeader(const GWEN_XMLNODE *n){
 void GWEN_XMLNode_AddHeader(GWEN_XMLNODE *n, GWEN_XMLNODE *nh){
   assert(n);
   assert(nh);
-  GWEN_XMLNode_add(nh, &(n->header));
+  GWEN_XMLNode_List_Add(nh, n->headers);
 }
 
 
@@ -2621,15 +2614,14 @@ void GWEN_XMLNode_AddHeader(GWEN_XMLNODE *n, GWEN_XMLNODE *nh){
 void GWEN_XMLNode_DelHeader(GWEN_XMLNODE *n, GWEN_XMLNODE *nh){
   assert(n);
   assert(nh);
-  GWEN_XMLNode_del(nh, &(n->header));
+  GWEN_XMLNode_List_Del(nh);
 }
 
 
 
 void GWEN_XMLNode_ClearHeaders(GWEN_XMLNODE *n){
   assert(n);
-  GWEN_XMLNode_freeAll(n->header);
-  n->header=0;
+  GWEN_XMLNode_List_Clear(n->headers);
 }
 
 
@@ -2654,7 +2646,7 @@ int GWEN_XML_AddNameSpace(GWEN_STRINGLIST2 *sl,
   GWEN_Buffer_AppendByte(nbuf, ':');
   GWEN_Buffer_AppendString(nbuf, name);
   rv=GWEN_StringList2_AppendString(sl, GWEN_Buffer_GetStart(nbuf), 0,
-                                   GWEN_StringList2_IntertModeNoDouble);
+                                   GWEN_StringList2_IntertMode_NoDouble);
   GWEN_Buffer_free(nbuf);
   return rv;
 }
@@ -2781,8 +2773,8 @@ int GWEN_XMLNode__CheckNameSpaceDecls1(GWEN_XMLNODE *n,
         }
         else {
           /* current namespace changed */
-          free(localNameSpace);
-          localNameSpace=strdup(pr->value);
+          GWEN_Memory_dealloc(localNameSpace);
+          localNameSpace=GWEN_Memory_strdup(pr->value);
         }
       }
       else if (currentNameSpace) {
@@ -2793,14 +2785,14 @@ int GWEN_XMLNode__CheckNameSpaceDecls1(GWEN_XMLNODE *n,
         }
         else {
           /* current namespace changed */
-          free(localNameSpace);
-          localNameSpace=strdup(pr->value);
+          GWEN_Memory_dealloc(localNameSpace);
+          localNameSpace=GWEN_Memory_strdup(pr->value);
         }
       }
       else {
         /* set current namespace */
-        free(localNameSpace);
-        localNameSpace=strdup(pr->value);
+        GWEN_Memory_dealloc(localNameSpace);
+        localNameSpace=GWEN_Memory_strdup(pr->value);
       }
     }
     else if (strncasecmp(pr->name, "xmlns:", 6)==0) {
@@ -2853,13 +2845,13 @@ int GWEN_XMLNode__CheckNameSpaceDecls1(GWEN_XMLNODE *n,
                                           localNameSpace?localNameSpace:
                                           currentNameSpace);
     if (rv) {
-      free(localNameSpace);
+      GWEN_Memory_dealloc(localNameSpace);
       return rv;
     }
     nn=GWEN_XMLNode_GetNextTag(nn);
   }
 
-  free(localNameSpace);
+  GWEN_Memory_dealloc(localNameSpace);
   return 0;
 }
 
