@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: xsd_write.c,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: xsd_write.c,v 1.9 2005/04/26 11:30:21 cstim Exp $
+ cvs         : $Id: xsd_write.c 1024 2006-03-27 12:25:01Z cstim $
  begin       : Sat Jun 28 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -189,7 +189,7 @@ int GWEN_XSD__ValidateElementData(GWEN_XSD_ENGINE *e,
     dcount=0;
     p=GWEN_Buffer_GetStart(vbuf);
     while(*p) {
-      if (isdigit(*p))
+      if (isdigit((int)*p))
         dcount++;
       p++;
     }
@@ -220,7 +220,7 @@ int GWEN_XSD__ValidateElementData(GWEN_XSD_ENGINE *e,
     p=strchr(GWEN_Buffer_GetStart(vbuf), '.');
     if (p) {
       while(*(++p)) {
-        if (isdigit(*p))
+        if (isdigit((int)*p))
           dcount++;
       }
     }
