@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: version.h.in,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: version.h.in,v 1.6 2005/08/17 14:09:29 aquamaniac Exp $
+ cvs         : $Id: version.h.in 832 2005-08-17 14:09:29Z aquamaniac $
  begin       : Wed Sep 24 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -29,17 +29,17 @@
 #ifndef GWENHYWFAR_VERSION_H
 #define GWENHYWFAR_VERSION_H
 
-#define GWEN_VERSION_MAJOR 1
-#define GWENHYWFAR_VERSION_MAJOR 1
-#define GWENHYWFAR_VERSION_MINOR 19
-#define GWENHYWFAR_VERSION_PATCHLEVEL 2
+#define GWEN_VERSION_MAJOR 2
+#define GWENHYWFAR_VERSION_MAJOR 2
+#define GWENHYWFAR_VERSION_MINOR 2
+#define GWENHYWFAR_VERSION_PATCHLEVEL 0
 #define GWENHYWFAR_VERSION_BUILD 0
 #define GWENHYWFAR_VERSION_TAG "stable"
-#define GWENHYWFAR_VERSION_FULL_STRING "1.19.2.0stable"
-#define GWENHYWFAR_VERSION_STRING "1.19.2"
+#define GWENHYWFAR_VERSION_FULL_STRING "2.2.0-stable-r0"
+#define GWENHYWFAR_VERSION_STRING "2.2.0"
 
-#define GWENHYWFAR_SO_EFFECTIVE 17
-#define GWENHYWFAR_SO_EFFECTIVE_STR "17"
+#define GWENHYWFAR_SO_EFFECTIVE 38
+#define GWENHYWFAR_SO_EFFECTIVE_STR "38"
 
 
 #endif /* GWENHYWFAR_VERSION_H */
